"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _ConversionTable = _interopRequireDefault(require("./ConversionTable"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

// Regex to extract quantity and type of base measurement
// Expects `<quantity> <type>` - e.g. 12.5 cups
// Important capture groups: (1-based index)
//  1: Decimal or integer quantity (e.g. 12.5)
//  3: Singular measurement type (e.g. 'cup')
var BASE_MEASUREMENT_TEST = /^([0-9]+([,.][0-9]+)?)\s([a-zA-Z]+?)(s\b|\b)/;

var Quantity =
/*#__PURE__*/
function () {
  function Quantity(baseMeasurement) {
    _classCallCheck(this, Quantity);

    // Input measurement must conform to the regex described above
    if (!BASE_MEASUREMENT_TEST.test(baseMeasurement)) {
      throw new Error('The input measurement must be in the format `<quantity> <unit>` where quantity is an integer or decimal.');
    } // Execute regex on the lowercase input


    var measurementFields = BASE_MEASUREMENT_TEST.exec(baseMeasurement.toLowerCase()); // Store the raw input along with extracted quantity and units

    this.baseMeasurement = {
      raw: baseMeasurement,
      quantity: measurementFields[1],
      unit: measurementFields[3]
    };
  }

  _createClass(Quantity, [{
    key: "toOunces",
    value: function toOunces() {
      return this.convertTo('ounce').toNumber();
    }
  }, {
    key: "toCups",
    value: function toCups() {
      return this.convertTo('cup').toNumber();
    }
  }, {
    key: "toTablespoons",
    value: function toTablespoons() {
      return this.convertTo('tablespoon').toNumber();
    }
  }, {
    key: "toTeaspoons",
    value: function toTeaspoons() {
      return this.convertTo('teaspoon').toNumber();
    }
  }, {
    key: "convertTo",
    value: function convertTo(toUnit) {
      return new Quantity("".concat(_ConversionTable.default.getConversionFactor(this.baseMeasurement.unit, toUnit) * this.baseMeasurement.quantity, " ").concat(toUnit));
    }
  }, {
    key: "toNumber",
    value: function toNumber() {
      return Number(this.baseMeasurement.quantity);
    }
  }, {
    key: "toString",
    value: function toString() {
      // Pluralize the unit if necessary
      // TODO: Determine if pluralization makes sense. E.g. if the unit is 'oz,' it doesn't make
      // sense to add an 's' even if the quantity is greater than 1.
      var unitString = this.baseMeasurement.quantity > 1 ? "".concat(this.baseMeasurement.unit, "s") : this.baseMeasurement.unit;
      return "".concat(this.baseMeasurement.quantity, " ").concat(unitString);
    }
  }]);

  return Quantity;
}();

exports.default = Quantity;